import json
import requests
from typing import Literal
from exceptions import APIException
from models import Bot, Keypad, Chat, Update


def _send_request(token: str, method: str, data: dict) -> dict:
    url = f'https://messengerg2b1.iranlms.ir/v3/{token}/{method}'
    res = requests.post(url=url, data=json.dumps(data), timeout=5)
    body = json.loads(res.text or '{}')
    if res.status_code != 200 or body.get('status') != 'OK':
        print(f'{res.status_code = }')
        raise APIException(f'APIException(status_code: {res.status_code}, body: {body}')
    return body['data']


def get_me(token: str) -> Bot:
    bot = _send_request(token=token, method='getMe', data={})
    return Bot(**bot)


def send_message(
        token: str,
        chat_id: str,
        text: str,
        chat_keypad: Keypad | None = None,
        disable_notification: bool = False,
        inline_keypad: Keypad | None = None,
        reply_to_message_id: str | None = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
) -> str:
    data = {
        'text': text,
        'chat_id': chat_id,
        'disable_notification': disable_notification
    }
    if chat_keypad:
        data['chat_keypad'] = chat_keypad.dict()
        data['chat_keypad_type'] = chat_keypad_type
    if inline_keypad:
        data['inline_keypad'] = inline_keypad.dict()
    if reply_to_message_id:
        data['reply_to_message_id'] = reply_to_message_id

    res = _send_request(token=token, method='sendMessage', data=data)
    message_id = res['message_id']
    return message_id


def send_poll(
        token: str,
        chat_id: str,
        question: str,
        options: list[str],
        chat_keypad: Keypad | None = None,
        disable_notification: bool = False,
        inline_keypad: Keypad | None = None,
        reply_to_message_id: str | None = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
) -> str:
    data = {
        'chat_id': chat_id,
        'question': question,
        'options': options,
        'chat_keypad': chat_keypad,
        'disable_notification': disable_notification,
        'inline_keypad': inline_keypad,
        'reply_to_message_id': reply_to_message_id,
        'chat_keypad_type': chat_keypad_type,
    }

    res = _send_request(token=token, method='sendPoll', data=data)
    message_id = res['message_id']
    return message_id


def send_location(
        token: str,
        chat_id: str,
        latitude: str,
        longitude: str,
        chat_keypad: Keypad | None = None,
        disable_notification: bool = False,
        inline_keypad: Keypad | None = None,
        reply_to_message_id: str | None = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
) -> str:
    data = {
        'chat_id': chat_id,
        'latitude': latitude,
        'longitude': longitude,
        'chat_keypad': chat_keypad,
        'disable_notification': disable_notification,
        'inline_keypad': inline_keypad,
        'reply_to_message_id': reply_to_message_id,
        'chat_keypad_type': chat_keypad_type,
    }

    res = _send_request(token=token, method='sendPoll', data=data)
    message_id = res['message_id']
    return message_id


def send_sticker(
        token: str,
        chat_id: str,
        sticker_id: str,
        chat_keypad: Keypad | None = None,
        disable_notification: bool = False,
        inline_keypad: Keypad | None = None,
        reply_to_message_id: str | None = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,

) -> str:
    data = {
        'chat_id': chat_id,
        'sticker_id': sticker_id,
        'chat_keypad': chat_keypad,
        'disable_notification': disable_notification,
        'inline_keypad': inline_keypad,
        'reply_to_message_id': reply_to_message_id,
        'chat_keypad_type': chat_keypad_type,
    }

    res = _send_request(token=token, method='sendSticker', data=data)
    message_id = res['message_id']
    return message_id


def send_contact(
        token: str,
        chat_id: str,
        first_name: str,
        last_name: str,
        phone_number: str,
        chat_keypad: Keypad | None = None,
        disable_notification: bool = False,
        inline_keypad: Keypad | None = None,
        reply_to_message_id: str | None = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,

) -> str:
    data = {
        'chat_id': chat_id,
        'last_name': last_name,
        'first_name': first_name,
        'chat_keypad': chat_keypad,
        'phone_number': phone_number,
        'inline_keypad': inline_keypad,
        'chat_keypad_type': chat_keypad_type,
        'reply_to_message_id': reply_to_message_id,
        'disable_notification': disable_notification,
    }

    res = _send_request(token=token, method='sendContact', data=data)
    message_id = res['message_id']
    return message_id


def get_chat(token: str, chat_id: str) -> Chat:
    data = {'chat_id': chat_id}
    chat = _send_request(token=token, method='getChat', data=data)
    return Chat(**chat)


def get_updates(token: str, limit: int, offset_id: str) -> tuple[list[Update], str]:
    data = {
        'limit': limit,
    }
    if offset_id:
        data['offset_id'] = offset_id

    res = _send_request(token=token, method='getUpdates', data=data)
    return [Update(**update) for update in res['updates']], res['next_offset_id']


def forward_message(
        token: str,
        from_chat_id: str,
        message_id: str,
        to_chat_id: str,
        disable_notification: bool = False
) -> str:
    data = {
        'from_chat_id': from_chat_id,
        'message_id': message_id,
        'to_chat_id': to_chat_id,
        'disable_notification': disable_notification,
    }
    res = _send_request(token=token, method='forwardMessage', data=data)
    return res['new_message_id']


def edit_message_text(token: str, chat_id: str, message_id: str, text: str) -> None:
    data = {
        'text': text,
        'chat_id': chat_id,
        'message_id': message_id,
    }
    _send_request(token=token, method='editMessageText', data=data)


def edit_message_keypad(token: str, chat_id: str, message_id: str, inline_keypad: Keypad) -> None:
    data = {
        'chat_id': chat_id,
        'message_id': message_id,
        'inline_keypad': inline_keypad,
    }
    _send_request(token=token, method='editMessageKeypad', data=data)


def delete_message(token: str, chat_id: str, message_id: str) -> None:
    data = {
        'chat_id': chat_id,
        'message_id': message_id,
    }
    _send_request(token=token, method='deleteMessage', data=data)


def send_file(
        token: str,
        chat_id: str,
        file_id: str,
        chat_keypad: Keypad | None = None,
        disable_notification: bool = False,
        inline_keypad: Keypad | None = None,
        reply_to_message_id: str | None = None,
        chat_keypad_type: Literal[None, 'New', 'Remove'] = None,
) -> str:
    data = {
        'chat_id': chat_id,
        'file_id': file_id,
        'chat_keypad': chat_keypad,
        'disable_notification': disable_notification,
        'inline_keypad': inline_keypad,
        'reply_to_message_id': reply_to_message_id,
        'chat_keypad_type': chat_keypad_type,

    }
    res = _send_request(token=token, method='sendFile', data=data)
    return res['message_id']


def request_send_file(token: str, type: Literal['File', 'Image', 'Voice', 'Music', 'Gif']) -> str:
    data = {
        'type': type,
    }
    res = _send_request(token=token, method='requestSendFile', data=data)
    return res['upload_url']


def upload():
    ...


def get_file():
    ...


def set_commands():
    ...


def edit_chat_keypad():
    ...


def update_bot_endpoint():
    ...


def send_payment_message():
    ...


def get_payment_status():
    ...


def settle_payment():
    ...


def reverse_payment():
    ...




import os
import re
from setuptools import setup, find_packages


def rubika_version() -> str:
    with open(os.path.join('rubika_bot/__init__.py')) as f:
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", f.read()).group(1)


_version = rubika_version()
_long_description = open('README.md').read()

setup(
    name='rubika-bot',
    version=_version,
    python_requires='>=3.10',
    author='Rubika Team',
    author_email='A.RajabNezhad@rubika.ir',
    keywords=['rubika', 'bot', 'rubika-bot'],
    url='https://github.com/alirn76/rubika-bot',
    description='Rubika Bot Library',
    long_description=_long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.10',
    ],
    packages=find_packages(where='.', exclude=(), include=('*',)),
    install_requires=['requests', 'pydantic'],
)








from datetime import datetime
from pydantic import BaseModel
from typing import Literal


class File(BaseModel):
    file_id: str
    file_name: str
    size: str


class Chat(BaseModel):
    chat_id: str
    chat_type: Literal['User', 'Bot', 'Group', 'Channel']
    user_id: str
    first_name: str
    last_name: str
    title: str
    username: str


class ForwardedFrom(BaseModel):
    type_from: Literal['User', 'Channel', 'Bot']
    message_id: str
    from_chat_id: str
    from_sender_id: str


class PaymentStatus(BaseModel):
    payment_id: str
    status: Literal['Paid', 'NotPaid']


class MessageTextUpdate(BaseModel):
    message_id: str
    text: str


class Bot(BaseModel):
    bot_id: str
    bot_title: str
    avatar: File
    description: str
    username: str
    start_message: datetime
    share_url: str


class BotCommand(BaseModel):
    command: str
    description: str


class Sticker(BaseModel):
    sticker_id: str
    file: File
    emoji_character: str


class ContactMessage(BaseModel):
    phone_number: str
    first_name: str
    last_name: str


class PollStatus(BaseModel):
    state: Literal['Open', 'Closed']
    selection_index: int
    percent_vote_options: list[int]
    total_vote: int
    show_total_votes: bool


class Poll(BaseModel):
    question: str
    options: list[str]
    poll_status: PollStatus


class Location(BaseModel):
    longitude: str
    latitude: str


class LiveLocation(BaseModel):
    start_time: datetime
    live_period: int  # In Second
    current_location: Location
    user_id: str
    status: Literal['Stopped', 'Live']
    last_update_time: datetime


class ButtonSelectionItem(BaseModel):
    text: str
    image_url: str
    type: Literal['TextOnly', 'TextImgThu', 'TextImgBig']


class ButtonSelection(BaseModel):
    selection_id: str
    search_type: Literal[None, 'Local', 'Api']
    get_type: Literal['Local', 'Api']
    items: list[ButtonSelectionItem]
    is_multi_selection: bool
    columns_count: str
    title: str


class ButtonCalendar(BaseModel):
    default_value: str
    type: Literal['DatePersian', 'DateGregorian']
    min_year: str
    max_year: str
    title: str


class ButtonNumberPicker(BaseModel):
    min_value: int
    max_value: int
    default_value: int | None
    title: str


class ButtonStringPicker(BaseModel):
    items: list[str]
    default_value: str
    title: str


class ButtonTextbox(BaseModel):
    type_line: Literal['SingleLine', 'MultiLine']
    type_keypad: Literal['String', 'Number']
    place_holder: str
    title: str
    default_value: str


class ButtonLocation(BaseModel):
    default_pointer_location: Location
    default_map_location: Location
    type: Literal['Picker', 'View']
    title: str
    location_image_url: str


class AuxData(BaseModel):
    start_id: str | None
    button_id: str | None


class Button(BaseModel):
    id: str
    type: Literal['Simple', 'Selection', 'Calendar', 'NumberPicker', 'StringPicker', 'Location', 'Payment',
                  'CameraImage', 'CameraVideo', 'GalleryImage', 'GalleryVideo', 'File', 'Audio', 'RecordAudio',
                  'MyPhoneNumber', 'MyLocation', 'Textbox', 'Link', 'AskMyPhoneNumber', 'AskLocation', 'Barcode']
    button_text: str
    button_selection: ButtonSelection | None
    button_calendar: ButtonCalendar | None
    button_number_picker: ButtonNumberPicker | None
    button_string_picker: ButtonStringPicker | None
    button_location: ButtonLocation | None
    button_textbox: ButtonTextbox | None
    # button_link: Link | None


class KeypadRow(BaseModel):
    buttons: list[Button]


class Keypad(BaseModel):
    rows: list[KeypadRow]
    resize_keyboard: bool | None
    on_time_keyboard: bool | None


class MessageKeypadUpdate(BaseModel):
    message_id: str
    inline_keypad: Keypad


class Message(BaseModel):
    message_id: str
    text: str
    time: int
    is_edited: bool
    sender_type: Literal['User', 'Bot']
    sender_id: str

    aux_data: AuxData | None
    file: File | None
    reply_to_message_id: str | None
    forwarded_from: ForwardedFrom | None
    forwarded_no_link: str | None  # TODO: type? from_title ?
    location: Location | None
    sticker: Sticker | None
    contact_message: ContactMessage | None
    poll: Poll | None
    live_location: LiveLocation | None


class Update(BaseModel):
    type: Literal['UpdatedMessage', 'NewMessage', 'RemovedMessage', 'StartedBot', 'StoppedBot', 'UpdatedPayment']
    chat_id: str
    removed_message_id: str | None
    new_message: Message
    updated_message: Message | None
    updated_payment: PaymentStatus | None


class InlineMessage(BaseModel):
    sender_id: str
    text: str
    file: File
    location: Location | None
    aux_data: AuxData
    message_id: str
    chat_id: str
